#ifndef ENTITIES_HPP
#define ENTITIES_HPP


#include <string>


// sEntity
struct sEntity
{
  enum EntityType {
    NONE,
    WARP,
    PERSON,
    TRIGGER,
    DOODAD,
  };

  sEntity();
  EntityType GetEntityType() const;

  int x;  // x coordinate (pixel)
  int y;  // y coordinate (pixel)
  int layer;

protected:
  EntityType m_EntityType;
};


// sWarp
struct sWarpEntity : sEntity
{
  sWarpEntity();

  std::string destination_map;
  int         destination_x;
  int         destination_y;
  int         destination_layer;
  int         fade;
};


// sPerson
struct sPersonEntity : sEntity
{
  sPersonEntity();

  enum { NONE = 0, RANDOM = 1, REPEATING = 2 };
  enum { TEXT = 0, FILE = 1 };

  std::string name;
  std::string spriteset;

  // movement
  int movement_type;
  int walk_speed;
  int walk_duration;  // number of tiles to walk
  int walk_delay;

  // dialog
  int         dialogue_type;   // 0 = embedded, 1 = file
  std::string dialogue;        // if dialogue_type is 0, this is just the dialogue text
                               // if dialogue_type is 1, this is a filename with the text
};


// sTrigger
struct sTriggerEntity : sEntity
{
  sTriggerEntity();

  std::string function;
};


// sDoodad
struct sDoodadEntity : sEntity
{
  sDoodadEntity();

  enum { NONE = 0, DOOR = 1, CHEST = 2 };

  std::string spriteset_file;
  std::string function;
  bool        obstructive;
  int         activation_method;
};


#endif
