#include <stddef.h>
#include "smap.hpp"


////////////////////////////////////////////////////////////////////////////////

SMAP::SMAP()
{
}

////////////////////////////////////////////////////////////////////////////////

SMAP::~SMAP()
{
  for (int i = 0; i < m_Tiles.size(); i++)
    DestroyImage(m_Tiles[i]);
}

////////////////////////////////////////////////////////////////////////////////

bool
SMAP::Load(FILE* file)
{
  bool result = m_Map.LoadFromFile(file);
  fclose(file);

  if (result == false)
    return false;

  if (m_Map.GetTileset().GetNumTiles() == 0)
    return false;

  InitializeDelayMap();

  m_Tiles.resize(m_Map.GetTileset().GetNumTiles());
  for (int i = 0; i < m_Map.GetTileset().GetNumTiles(); i++)
  {
    sTile& tile = m_Map.GetTileset().GetTile(i);
    m_Tiles[i] = CreateImage(tile.GetWidth(), tile.GetHeight(), tile.GetPixels());
  }

  return true;
}

////////////////////////////////////////////////////////////////////////////////

void
SMAP::UpdateAnimations()
{

  sTileset& tileset = m_Map.GetTileset();

  // go through each tile and update the timer on it
  // if the tile should be switched, it is
  for (int il = 0; il < m_Map.GetNumLayers(); il++)
  {
    sLayer& layer = m_Map.GetLayer(il);
    for (int iy = 0; iy < layer.GetHeight(); iy++)
      for (int ix = 0; ix < layer.GetWidth(); ix++)
      {
        int oldtile = layer.GetTile(ix, iy);
        if (tileset.GetTile(oldtile).IsAnimated())
        {
          int offset = iy * layer.GetWidth() + ix;
          if (--m_DelayMap[il][offset] <= 0)
          {
            int newtile = tileset.GetTile(oldtile).GetNextTile();

            // change the map's tile and update the delay map
            layer.SetTile(ix, iy, newtile);
            m_DelayMap[il][offset] = tileset.GetTile(newtile).GetDelay();
          }
        }
      }
  }

}

////////////////////////////////////////////////////////////////////////////////


void
SMAP::InitializeDelayMap()
{
  sTileset& tileset = m_Map.GetTileset();

  m_DelayMap.resize(0);
  m_DelayMap.resize(m_Map.GetNumLayers());

  for (int i = 0; i < m_Map.GetNumLayers(); i++)
  {
    const sLayer& layer = m_Map.GetLayer(i);
    int width  = layer.GetWidth();
    int height = layer.GetHeight();
    m_DelayMap[i].resize(width * height);

    for (int j = 0; j < width * height; j++)
    {
      int tile = m_Map.GetLayer(i).GetTile(j % width, j / width);
      m_DelayMap[i][j] = tileset.GetTile(tile).GetDelay();
    }
  }
}


////////////////////////////////////////////////////////////////////////////////
